searchNodes=[{"doc":"This module provides the main API to interface with Redis. Overview start_link/2 starts a process that connects to Redis. Each Elixir process started with this function maps to a client TCP connection to the specified Redis server. The architecture is very simple: when you issue commands to Redis (via command/3 or pipeline/3), the Redix process sends the command to Redis right away and is immediately able to send new commands. When a response arrives from Redis, only then the Redix process replies to the caller with the response. This pattern avoids blocking the Redix process for each request (until a response arrives), increasing the performance of this driver. Reconnections Redix tries to be as resilient as possible: it tries to recover automatically from most network errors. If there&#39;s a network error when sending data to Redis or if the connection to Redis drops, Redix tries to reconnect. The first reconnection attempt will happen after a fixed time interval; if this attempt fails, reconnections are attempted until successful, and the time interval between reconnections is increased exponentially. Some aspects of this behaviour can be configured; see start_link/2 and the &quot;Reconnections&quot; page in the docs for more information. Sentinel Note: support for Redis Sentinel is still experimental. It works, but the API might change a little bit and the design might be revisited. Redix supports Redis Sentinel by passing a :sentinel option to start_link/1 (or start_link/2) instead of :host and :port. In :sentinel, you&#39;ll specify a list of sentinel nodes to try when connecting and the name of a primary group (see start_link/1 for more detailed information on these options). When connecting, Redix will attempt to connect to each of the specified sentinels in the given order. When it manages to connect to a sentinel, it will ask that sentinel for the address of the primary for the given primary group. Then, it will connect to that primary and ask it for confirmation that it is indeed a primary. If anything in this process doesn&#39;t go right, the next sentinel in the list will be tried. All of this happens in case of disconnections as well. If there&#39;s a disconnection, the whole process of asking sentinels for a primary is executed again. You should only care about Redis Sentinel when starting a Redix connection: once started, using the connection will be exactly the same as the non-sentinel scenario. Transactions or pipelining? Pipelining and transactions have things in common but they&#39;re fundamentally different. With a pipeline, you&#39;re sending all commands in the pipeline at once on the connection to Redis. This means Redis receives all commands at once, but the Redis server is not guaranteed to process all those commands at once. On the other hand, a MULTI/EXEC transaction guarantees that when EXEC is called all the queued commands in the transaction are executed atomically. However, you don&#39;t need to send all the commands in the transaction at once. If you want to combine pipelining with MULTI/EXEC transactions, use transaction_pipeline/3. Skipping replies Redis provides commands to control whether you want replies to your commands or not. These commands are CLIENT REPLY ON, CLIENT REPLY SKIP, and CLIENT REPLY OFF. When you use CLIENT REPLY SKIP, only the command that follows will not get a reply. When you use CLIENT REPLY OFF, all the commands that follow will not get replies until CLIENT REPLY ON is issued. Redix does not support these commands directly because they would change the whole state of the connection. To skip replies, use noreply_pipeline/3 or noreply_command/3. Skipping replies is useful to improve performance when you want to issue many commands but are not interested in the responses to those commands. SSL Redix supports SSL by passing ssl: true in start_link/1. You can use the :socket_opts option to pass options that will be used by the SSL socket, like certificates. If the CAStore dependency is available, Redix will pick up its CA certificate store file automatically. You can select a different CA certificate store by passing in the :cacertfile or :cacerts socket options. If the server uses a self-signed certificate, such as for testing purposes, disable certificate verification by passing verify: :verify_none in the socket options. Some Redis servers, notably Amazon ElastiCache, use wildcard certificates that require additional socket options for succesful verification (requires OTP 21.0 or later): Redix.start_link( host: &quot;example.com&quot;, port: 9999, ssl: true, socket_opts: [ customize_hostname_check: [ match_fun: :public_key.pkix_verify_hostname_match_fun(:https) ] ] ) Telemetry Redix uses Telemetry for instrumentation and logging. See Redix.Telemetry.","ref":"Redix.html","title":"Redix","type":"module"},{"doc":"Returns a child spec to use Redix in supervision trees. To use Redix with the default options (same as calling start_link()): children = [ Redix, # ... ] You can pass options: children = [ {Redix, host: &quot;redix.example.com&quot;, name: :redix}, # ... ] You can also pass a URI: children = [ {Redix, &quot;redis://redix.example.com:6380&quot;} ] If you want to pass both a URI and options, you can do it by passing a tuple with the URI as the first element and the list of options (make sure it has brackets around if using literals) as the second element: children = [ {Redix, {&quot;redis://redix.example.com&quot;, [name: :redix]}} ]","ref":"Redix.html#child_spec/1","title":"Redix.child_spec/1","type":"function"},{"doc":"Issues a command on the Redis server. This function sends command to the Redis server and returns the response returned by Redis. pid must be the pid of a Redix connection. command must be a list of strings making up the Redis command and its arguments. The return value is {:ok, response} if the request is successful and the response is not a Redis error. {:error, reason} is returned in case there&#39;s an error in the request (such as losing the connection to Redis in between the request). reason can also be a Redix.Error exception in case Redis is reachable but returns an error (such as a type error). If the given command is an empty command ([]), an ArgumentError exception is raised. Options :timeout - (integer or :infinity) request timeout (in milliseconds). Defaults to 5000. If the Redis server doesn&#39;t reply within this timeout, {:error, %Redix.ConnectionError{reason: :timeout}} is returned. Examples iex&gt; Redix.command(conn, [&quot;SET&quot;, &quot;mykey&quot;, &quot;foo&quot;]) {:ok, &quot;OK&quot;} iex&gt; Redix.command(conn, [&quot;GET&quot;, &quot;mykey&quot;]) {:ok, &quot;foo&quot;} iex&gt; Redix.command(conn, [&quot;INCR&quot;, &quot;mykey&quot;]) {:error, &quot;ERR value is not an integer or out of range&quot;} If Redis goes down (before a reconnection happens): iex&gt; {:error, error} = Redix.command(conn, [&quot;GET&quot;, &quot;mykey&quot;]) iex&gt; error.reason :closed","ref":"Redix.html#command/3","title":"Redix.command/3","type":"function"},{"doc":"Issues a command on the Redis server, raising if there&#39;s an error. This function works exactly like command/3 but: if the command is successful, then the result is returned directly (not wrapped in a {:ok, result} tuple). if there&#39;s a Redis error or a connection error, a Redix.Error or Redix.ConnectionError error is raised. This function accepts the same options as command/3. Options :timeout - (integer or :infinity) request timeout (in milliseconds). Defaults to 5000. If the Redis server doesn&#39;t reply within this timeout, {:error, %Redix.ConnectionError{reason: :timeout}} is returned. Examples iex&gt; Redix.command!(conn, [&quot;SET&quot;, &quot;mykey&quot;, &quot;foo&quot;]) &quot;OK&quot; iex&gt; Redix.command!(conn, [&quot;INCR&quot;, &quot;mykey&quot;]) ** (Redix.Error) ERR value is not an integer or out of range If Redis goes down (before a reconnection happens): iex&gt; Redix.command!(conn, [&quot;GET&quot;, &quot;mykey&quot;]) ** (Redix.ConnectionError) :closed","ref":"Redix.html#command!/3","title":"Redix.command!/3","type":"function"},{"doc":"Same as command/3 but tells the Redis server to not return a response. This function is useful when you want to send a command but you don&#39;t care about the response. Since the response is not returned, the return value of this function in case the command is successfully sent to Redis is :ok. Not receiving a response means saving traffic on the network and memory allocation for the response. See also noreply_pipeline/3. Options :timeout - (integer or :infinity) request timeout (in milliseconds). Defaults to 5000. If the Redis server doesn&#39;t reply within this timeout, {:error, %Redix.ConnectionError{reason: :timeout}} is returned. Examples iex&gt; Redix.noreply_command(conn, [&quot;INCR&quot;, &quot;mykey&quot;]) :ok iex&gt; Redix.command(conn, [&quot;GET&quot;, &quot;mykey&quot;]) {:ok, &quot;1&quot;}","ref":"Redix.html#noreply_command/3","title":"Redix.noreply_command/3","type":"function"},{"doc":"Same as noreply_command/3 but raises in case of errors.","ref":"Redix.html#noreply_command!/3","title":"Redix.noreply_command!/3","type":"function"},{"doc":"Issues a pipeline of commands to the Redis server, asking the server to not send responses. This function is useful when you want to issue commands to the Redis server but you don&#39;t care about the responses. For example, you might want to set a bunch of keys but you don&#39;t care for a confirmation that they were set. In these cases, you can save bandwith by asking Redis to not send replies to your commands. Since no replies are sent back, this function returns :ok in case there are no network errors, or {:error, reason} otherwise Options :timeout - (integer or :infinity) request timeout (in milliseconds). Defaults to 5000. If the Redis server doesn&#39;t reply within this timeout, {:error, %Redix.ConnectionError{reason: :timeout}} is returned. Examples iex&gt; commands = [[&quot;INCR&quot;, &quot;mykey&quot;], [&quot;INCR&quot;, &quot;meykey&quot;]] iex&gt; Redix.noreply_pipeline(conn, commands) :ok iex&gt; Redix.command(conn, [&quot;GET&quot;, &quot;mykey&quot;]) {:ok, &quot;2&quot;}","ref":"Redix.html#noreply_pipeline/3","title":"Redix.noreply_pipeline/3","type":"function"},{"doc":"Same as noreply_pipeline/3 but raises in case of errors.","ref":"Redix.html#noreply_pipeline!/3","title":"Redix.noreply_pipeline!/3","type":"function"},{"doc":"Issues a pipeline of commands on the Redis server. commands must be a list of commands, where each command is a list of strings making up the command and its arguments. The commands will be sent as a single &quot;block&quot; to Redis, and a list of ordered responses (one for each command) will be returned. The return value is {:ok, results} if the request is successful, {:error, reason} otherwise. Note that {:ok, results} is returned even if results contains one or more Redis errors (Redix.Error structs). This is done to avoid having to walk the list of results (a O(n) operation) to look for errors, leaving the responsibility to the user. That said, errors other than Redis errors (like network errors) always cause the return value to be {:error, reason}. If commands is an empty list ([]) or any of the commands in commands is an empty command ([]) then an ArgumentError exception is raised right away. Pipelining is not the same as a transaction. For more information, see the module documentation. Options :timeout - (integer or :infinity) request timeout (in milliseconds). Defaults to 5000. If the Redis server doesn&#39;t reply within this timeout, {:error, %Redix.ConnectionError{reason: :timeout}} is returned. Examples iex&gt; Redix.pipeline(conn, [[&quot;INCR&quot;, &quot;mykey&quot;], [&quot;INCR&quot;, &quot;mykey&quot;], [&quot;DECR&quot;, &quot;mykey&quot;]]) {:ok, [1, 2, 1]} iex&gt; Redix.pipeline(conn, [[&quot;SET&quot;, &quot;k&quot;, &quot;foo&quot;], [&quot;INCR&quot;, &quot;k&quot;], [&quot;GET&quot;, &quot;k&quot;]]) {:ok, [&quot;OK&quot;, %Redix.Error{message: &quot;ERR value is not an integer or out of range&quot;}, &quot;foo&quot;]} If Redis goes down (before a reconnection happens): iex&gt; {:error, error} = Redix.pipeline(conn, [[&quot;SET&quot;, &quot;mykey&quot;, &quot;foo&quot;], [&quot;GET&quot;, &quot;mykey&quot;]]) iex&gt; error.reason :closed","ref":"Redix.html#pipeline/3","title":"Redix.pipeline/3","type":"function"},{"doc":"Issues a pipeline of commands to the Redis server, raising if there&#39;s an error. This function works similarly to pipeline/3, except: if there are no errors in issuing the commands (even if there are one or more Redis errors in the results), the results are returned directly (not wrapped in a {:ok, results} tuple). if there&#39;s a connection error then a Redix.ConnectionError exception is raised. For more information on why nothing is raised if there are one or more Redis errors (Redix.Error structs) in the list of results, look at the documentation for pipeline/3. This function accepts the same options as pipeline/3. Options :timeout - (integer or :infinity) request timeout (in milliseconds). Defaults to 5000. If the Redis server doesn&#39;t reply within this timeout, {:error, %Redix.ConnectionError{reason: :timeout}} is returned. Examples iex&gt; Redix.pipeline!(conn, [[&quot;INCR&quot;, &quot;mykey&quot;], [&quot;INCR&quot;, &quot;mykey&quot;], [&quot;DECR&quot;, &quot;mykey&quot;]]) [1, 2, 1] iex&gt; Redix.pipeline!(conn, [[&quot;SET&quot;, &quot;k&quot;, &quot;foo&quot;], [&quot;INCR&quot;, &quot;k&quot;], [&quot;GET&quot;, &quot;k&quot;]]) [&quot;OK&quot;, %Redix.Error{message: &quot;ERR value is not an integer or out of range&quot;}, &quot;foo&quot;] If Redis goes down (before a reconnection happens): iex&gt; Redix.pipeline!(conn, [[&quot;SET&quot;, &quot;mykey&quot;, &quot;foo&quot;], [&quot;GET&quot;, &quot;mykey&quot;]]) ** (Redix.ConnectionError) :closed","ref":"Redix.html#pipeline!/3","title":"Redix.pipeline!/3","type":"function"},{"doc":"Starts a connection to Redis. This function returns {:ok, pid} if the Redix process is started successfully. {:ok, pid} = Redix.start_link() The actual TCP connection to the Redis server may happen either synchronously, before start_link/2 returns, or asynchronously. This behaviour is decided by the :sync_connect option (see below). This function accepts one argument which can either be an string representing a URI or a keyword list of options. Using in supervision trees Redix supports child specs, so you can use it as part of a supervision tree: children = [ {Redix, host: &quot;redix.myapp.com&quot;, name: :redix} ] See child_spec/1 for more information. Using a Redis URI In case uri_or_opts is a Redis URI, it must be in the form: redis://[:password@]host[:port][/db] Here are some examples of valid URIs: redis://localhost redis://:secret@localhost:6397 redis://example.com:6380/1 Usernames before the password are ignored, so the these two URIs are equivalent: redis://:secret@localhost redis://myuser:secret@localhost The only mandatory thing when using URIs is the host. All other elements are optional and their default value can be found in the &quot;Options&quot; section below. Options Redis options The following options can be used to specify the parameters used to connect to Redis (instead of a URI as described above): :host - (string) the host where the Redis server is running. Defaults to &quot;localhost&quot;. :port - (positive integer) the port on which the Redis server is running. Defaults to 6379. :password - (string) the password used to connect to Redis. Defaults to nil, meaning no password is used. When this option is provided, all Redix does is issue an AUTH command to Redis in order to authenticate. :database - (non-negative integer or string) the database to connect to. Defaults to nil, meaning Redix doesn&#39;t connect to a specific database (the default in this case is database 0). When this option is provided, all Redix does is issue a SELECT command to Redis in order to select the given database. Connection options The following options can be used to tweak how the Redix connection behaves. :socket_opts - (list of options) this option specifies a list of options that are passed to the network layer when connecting to the Redis server. Some socket options (like :active or :binary) will be overridden by Redix so that it functions properly. Defaults to [] for TCP and [verify: :verify_peer, depth: 3] for SSL. If the CAStore dependency is available, the cacertfile option is added to the SSL options by default as well. :timeout - (integer) connection timeout (in milliseconds) also directly passed to the network layer. Defaults to 5000. :sync_connect - (boolean) decides whether Redix should initiate the TCP connection to the Redis server before or after returning from start_link/1. This option also changes some reconnection semantics; read the &quot;Reconnections&quot; page in the docs. :exit_on_disconnection - (boolean) if true, the Redix server will exit if it fails to connect or disconnects from Redis. Note that setting this option to true means that the :backoff_initial and :backoff_max options will be ignored. Defaults to false. :backoff_initial - (non-negative integer) the initial backoff time (in milliseconds), which is the time that the Redix process will wait before attempting to reconnect to Redis after a disconnection or failed first connection. See the &quot;Reconnections&quot; page in the docs for more information. :backoff_max - (positive integer) the maximum length (in milliseconds) of the time interval used between reconnection attempts. See the &quot;Reconnections&quot; page in the docs for more information. :log - (keyword list) a keyword list of {action, level} where level is the log level to use to log action. This option is deprecated in favor of Telemetry. See the &quot;Telemetry&quot; section in the module documentation. The possible actions and their default values are: :disconnection (defaults to :error) - logged when the connection to Redis is lost :failed_connection (defaults to :error) - logged when Redix can&#39;t establish a connection to Redis :reconnection (defaults to :info) - logged when Redix manages to reconnect to Redis after the connection was lost :name - Redix is bound to the same registration rules as a GenServer. See the GenServer documentation for more information. :ssl - (boolean) if true, connect through SSL, otherwise through TCP. The :socket_opts option applies to both SSL and TCP, so it can be used for things like certificates. See :ssl.connect/4. Defaults to false. :sentinel - (keyword list) options for using Redis Sentinel. If this option is provided, then the :host and :port option cannot be provided. For the available sentinel options, see the &quot;Sentinel options&quot; section below. :hibernate_after - (integer) if present, the Redix connection process awaits any message for the given number of milliseconds and if no message is received, the process goes into hibernation automatically (by calling :proc_lib.hibernate/3). See t::gen_statem.start_opt/0. Not present by default. :spawn_opt - (options) if present, its value is passed as options to the Redix connection process as in Process.spawn/4. See t::gen_statem.start_opt/0. Not present by default. :debug - (options) if present, the corresponding function in the :sys module is invoked. Not present by default. Sentinel options The following options can be used to configure the Redis Sentinel behaviour when connecting. These options should be passed in the :sentinel key in the connection options. For more information on support for Redis sentinel, see the Redix module documentation. :sentinels - (list) a list of sentinel addresses. Each element in this list is the address of a sentinel to be contacted in order to obtain the address of a primary. The address of a sentinel can be passed as a Redis URI (see the &quot;Using a Redis URI&quot; section above) or a keyword list with :host, :port, :password options (same as when connecting to a Redis instance direclty). Note that the password can either be passed in the sentinel address or globally -- see the :password option below. This option is required. :group - (binary) the name of the group that identifies the primary in the sentinel configuration. This option is required. :role - (:primary or :replica) if :primary, the connection will be established with the primary for the given group. If :replica, Redix will ask the sentinel for all the available replicas for the given group and try to connect to one of them at random. Defaults to :primary. :socket_opts - (list of options) the socket options that will be used when connecting to the sentinels. Defaults to []. :ssl - (boolean) if true, connect to the sentinels via through SSL, otherwise through TCP. The :socket_opts applies to both TCP and SSL, so it can be used for things like certificates. See :ssl.connect/4. Defaults to false. :timeout - (timeout) the timeout (in milliseconds or :infinity) that will be used to interact with the sentinels. This timeout will be used as the timeout when connecting to each sentinel and when asking sentinels for a primary. The Redis documentation suggests to keep this timeout short so that connection to Redis can happen quickly. :password - (string) if you don&#39;t want to specify a password for each sentinel you list, you can use this option to specify a password that will be used to authenticate on sentinels if they don&#39;t specify a password. This option is recommended over passing a password for each sentinel because in the future we might do sentinel auto-discovery, which means authentication can only be done through a global password that works for all sentinels. Examples iex&gt; Redix.start_link() {:ok, #PID&lt;...&gt;} iex&gt; Redix.start_link(host: &quot;example.com&quot;, port: 9999, password: &quot;secret&quot;) {:ok, #PID&lt;...&gt;} iex&gt; Redix.start_link(database: 3, name: :redix_3) {:ok, #PID&lt;...&gt;}","ref":"Redix.html#start_link/1","title":"Redix.start_link/1","type":"function"},{"doc":"Starts a connection to Redis. This is the same as start_link/1, but the URI and the options get merged. other_opts have precedence over the things specified in uri. Take this code: start_link(&quot;redis://localhost:6379&quot;, port: 6380) In this example, port 6380 will be used.","ref":"Redix.html#start_link/2","title":"Redix.start_link/2","type":"function"},{"doc":"Closes the connection to the Redis server. This function is synchronous and blocks until the given Redix connection frees all its resources and disconnects from the Redis server. timeout can be passed to limit the amout of time allowed for the connection to exit; if it doesn&#39;t exit in the given interval, this call exits. Examples iex&gt; Redix.stop(conn) :ok","ref":"Redix.html#stop/2","title":"Redix.stop/2","type":"function"},{"doc":"Executes a MULTI/EXEC transaction. Redis supports something akin to transactions. It works by sending a MULTI command, then some commands, and then an EXEC command. All the commands after MULTI are queued until EXEC is issued. When EXEC is issued, all the responses to the queued commands are returned in a list. Options :timeout - (integer or :infinity) request timeout (in milliseconds). Defaults to 5000. If the Redis server doesn&#39;t reply within this timeout, {:error, %Redix.ConnectionError{reason: :timeout}} is returned. Examples To run a MULTI/EXEC transaction in one go, use this function and pass a list of commands to use in the transaction: iex&gt; Redix.transaction_pipeline(conn, [[&quot;SET&quot;, &quot;mykey&quot;, &quot;foo&quot;], [&quot;GET&quot;, &quot;mykey&quot;]]) {:ok, [&quot;OK&quot;, &quot;foo&quot;]} Problems with transactions There&#39;s an inherent problem with Redix&#39;s architecture and MULTI/EXEC transaction. A Redix process is a single connection to Redis that can be used by many clients. If a client A sends MULTI and client B sends a command before client A sends EXEC, client B&#39;s command will be part of the transaction. This is intended behaviour, but it might not be what you expect. This is why transaction_pipeline/3 exists: this function wraps commands in MULTI/EXEC but sends all in a pipeline. Since everything is sent in the pipeline, it&#39;s sent at once on the connection and no commands can end up in the middle of the transaction. Running MULTI/EXEC transactions manually There are still some cases where you might want to start a transaction with MULTI, then send commands from different processes that you actively want to be in the transaction, and then send an EXEC to run the transaction. It&#39;s still fine to do this with command/3 or pipeline/3, but remember what explained in the section above. If you do this, do it in an isolated connection (open a new one if necessary) to avoid mixing things up.","ref":"Redix.html#transaction_pipeline/3","title":"Redix.transaction_pipeline/3","type":"function"},{"doc":"Executes a MULTI/EXEC transaction. Same as transaction_pipeline/3, but returns the result directly instead of wrapping it in an {:ok, result} tuple or raises if there&#39;s an error. Options :timeout - (integer or :infinity) request timeout (in milliseconds). Defaults to 5000. If the Redis server doesn&#39;t reply within this timeout, {:error, %Redix.ConnectionError{reason: :timeout}} is returned. Examples iex&gt; Redix.transaction_pipeline!(conn, [[&quot;SET&quot;, &quot;mykey&quot;, &quot;foo&quot;], [&quot;GET&quot;, &quot;mykey&quot;]]) [&quot;OK&quot;, &quot;foo&quot;]","ref":"Redix.html#transaction_pipeline!/3","title":"Redix.transaction_pipeline!/3","type":"function"},{"doc":"","ref":"Redix.html#t:command/0","title":"Redix.command/0","type":"type"},{"doc":"","ref":"Redix.html#t:connection/0","title":"Redix.connection/0","type":"type"},{"doc":"This module provides functions to work with the Redis binary protocol.","ref":"Redix.Protocol.html","title":"Redix.Protocol","type":"module"},{"doc":"Packs a list of Elixir terms to a Redis (RESP) array. This function returns an iodata (instead of a binary) because the packed result is usually sent to Redis through :gen_tcp.send/2 or similar. It can be converted to a binary with IO.iodata_to_binary/1. All elements of elems are converted to strings with to_string/1, hence this function supports encoding everything that implements String.Chars. Examples iex&gt; iodata = Redix.Protocol.pack([&quot;SET&quot;, &quot;mykey&quot;, 1]) iex&gt; IO.iodata_to_binary(iodata) &quot;*3\\r\\n$3\\r\\nSET\\r\\n$5\\r\\nmykey\\r\\n$1\\r\\n1\\r\\n&quot;","ref":"Redix.Protocol.html#pack/1","title":"Redix.Protocol.pack/1","type":"function"},{"doc":"Parses a RESP-encoded value from the given data. Returns {:ok, value, rest} if a value is parsed successfully, or a continuation in the form {:continuation, fun} if the data is incomplete. Examples iex&gt; Redix.Protocol.parse(&quot;+OK\\r\\ncruft&quot;) {:ok, &quot;OK&quot;, &quot;cruft&quot;} iex&gt; Redix.Protocol.parse(&quot;-ERR wrong type\\r\\n&quot;) {:ok, %Redix.Error{message: &quot;ERR wrong type&quot;}, &quot;&quot;} iex&gt; {:continuation, fun} = Redix.Protocol.parse(&quot;+OK&quot;) iex&gt; fun.(&quot;\\r\\n&quot;) {:ok, &quot;OK&quot;, &quot;&quot;}","ref":"Redix.Protocol.html#parse/1","title":"Redix.Protocol.parse/1","type":"function"},{"doc":"Parses n RESP-encoded values from the given data. Each element is parsed as described in parse/1. If an element can&#39;t be fully parsed or there are less than n elements encoded in data, then a continuation in the form of {:continuation, fun} is returned. Otherwise, {:ok, values, rest} is returned. If there&#39;s an error in decoding, a Redix.Protocol.ParseError exception is raised. Examples iex&gt; Redix.Protocol.parse_multi(&quot;+OK\\r\\n+COOL\\r\\n&quot;, 2) {:ok, [&quot;OK&quot;, &quot;COOL&quot;], &quot;&quot;} iex&gt; {:continuation, fun} = Redix.Protocol.parse_multi(&quot;+OK\\r\\n&quot;, 2) iex&gt; fun.(&quot;+OK\\r\\n&quot;) {:ok, [&quot;OK&quot;, &quot;OK&quot;], &quot;&quot;}","ref":"Redix.Protocol.html#parse_multi/2","title":"Redix.Protocol.parse_multi/2","type":"function"},{"doc":"","ref":"Redix.Protocol.html#t:on_parse/1","title":"Redix.Protocol.on_parse/1","type":"type"},{"doc":"","ref":"Redix.Protocol.html#t:redis_value/0","title":"Redix.Protocol.redis_value/0","type":"type"},{"doc":"Interface for the Redis pub/sub functionality. The rest of this documentation will assume the reader knows how pub/sub works in Redis and knows the meaning of the following Redis commands: SUBSCRIBE and UNSUBSCRIBE PSUBSCRIBE and PUNSUBSCRIBE PUBLISH Usage Each Redix.PubSub process is able to subcribe to/unsubscribe from multiple Redis channels/patterns, and is able to handle multiple Elixir processes subscribing each to different channels/patterns. A Redix.PubSub process can be started via Redix.PubSub.start_link/2; such a process holds a single TCP (or SSL) connection to the Redis server. Redix.PubSub has a message-oriented API. Subscribe operations are synchronous and return a reference that can then be used to match on all messages sent by the Redix.PubSub process. When Redix.PubSub registers a subscriptions, the subscriber process will receive a confirmation message: {:ok, pubsub} = Redix.PubSub.start_link() {:ok, ref} = Redix.PubSub.subscribe(pubsub, &quot;my_channel&quot;, self()) receive do message -&gt; message end #=&gt; {:redix_pubsub, ^pubsub, ^ref, :subscribed, %{channel: &quot;my_channel&quot;}} When the :subscribed message is received, it&#39;s guaranteed that the Redix.PubSub process has subscribed to the given channel. This means that after a subscription, messages published to a channel are delivered to all Elixir processes subscribed to that channel via Redix.PubSub: # Someone publishes &quot;hello&quot; on &quot;my_channel&quot; receive do message -&gt; message end #=&gt; {:redix_pubsub, ^pubsub, ^ref, :message, %{channel: &quot;my_channel&quot;, payload: &quot;hello&quot;}} It&#39;s advised to wait for the subscription confirmation for a channel before doing any other operation involving that channel. Note that unsubscription confirmations are delivered right away even if the Redix.PubSub process is still subscribed to the given channel: this is by design, as once a process is unsubscribed from a channel it won&#39;t receive messages anyways, even if the Redix.PubSub process still receives them. Messages are also delivered as a confirmation of an unsubscription as well as when the Redix.PubSub connection goes down. See the &quot;Messages&quot; section below. Messages Most of the communication with a PubSub connection is done via (Elixir) messages: the subscribers of these messages will be the processes specified at subscription time (in subscribe/3 or psubscribe/3). All Redix.PubSub messages have the same form: they&#39;re a five-element tuple that looks like this: {:redix_pubsub, pubsub_pid, subscription_ref, message_type, message_properties} where: pubsub_pid is the pid of the Redix.PubSub process that sent this message. subscription_ref is the reference returned by subscribe/3 or psubscribe/3. message_type is the type of this message, such as :subscribed for subscription confirmations, :message for pub/sub messages, and so on. message_properties is a map of data related to that that varies based on message_type. Given this format, it&#39;s easy to match on all Redix pub/sub messages for a subscription as {:redix_pubsub, _, ^subscription_ref, _, _}. List of possible message types and properties The following is a comprehensive list of possible message types alongside the properties that each can have. :subscribe - sent as confirmation of subscription to a channel (via subscribe/3 or after a disconnection and reconnection). One :subscribe message is received for every channel a process subscribed to. :subscribe messages have the following properties: :channel - the channel the process has been subscribed to. :psubscribe - sent as confirmation of subscription to a pattern (via psubscribe/3 or after a disconnection and reconnection). One :psubscribe message is received for every pattern a process subscribed to. :psubscribe messages have the following properties: :pattern - the pattern the process has been subscribed to. :unsubscribe - sent as confirmation of unsubscription from a channel (via unsubscribe/3). :unsubscribe messages are received for every channel a process unsubscribes from. :unsubscribe messages havethe following properties: :channel - the channel the process has unsubscribed from. :punsubscribe - sent as confirmation of unsubscription from a pattern (via unsubscribe/3). :unsubscribe messages are received for every pattern a process unsubscribes from. :unsubscribe messages havethe following properties: :pattern - the pattern the process has unsubscribed from. :message - sent to subscribers to a given channel when a message is published on that channel. :message messages have the following properties: :channel - the channel the message was published on :payload - the contents of the message :pmessage - sent to subscribers to a given pattern when a message is published on a channel that matches that pattern. :pmessage messages have the following properties: :channel - the channel the message was published on :pattern - the original pattern that matched the channel :payload - the contents of the message :disconnected messages - sent to all subscribers to all channels/patterns when the connection to Redis is interrupted. :disconnected messages have the following properties: :error - the reason for the disconnection, a Redix.ConnectionError exception struct (that can be raised or turned into a message through Exception.message/1). Reconnections Redix.PubSub tries to be resilient to failures: when the connection with Redis is interrupted (for whatever reason), it will try to reconnect to the Redis server. When a disconnection happens, Redix.PubSub will notify all clients subscribed to all channels with a {:redix_pubsub, pid, subscription_ref, :disconnected, _} message (more on the format of messages above). When the connection goes back up, Redix.PubSub takes care of actually re-subscribing to the appropriate channels on the Redis server and subscribers are notified with a {:redix_pubsub, pid, subscription_ref, :subscribed | :psubscribed, _} message, the same as when a client subscribes to a channel/pattern. Note that if exit_on_disconnection: true is passed to Redix.PubSub.start_link/2, the Redix.PubSub process will exit and not send any :disconnected messages to subscribed clients. Sentinel support Works exactly the same as for normal Redix connections. See the documentation for Redix for more information. Examples This is an example of a workflow using the PubSub functionality; it uses Redix as a Redis client for publishing messages. {:ok, pubsub} = Redix.PubSub.start_link() {:ok, client} = Redix.start_link() Redix.PubSub.subscribe(pubsub, &quot;my_channel&quot;, self()) #=&gt; {:ok, ref} # We wait for the subscription confirmation receive do {:redix_pubsub, ^pubsub, ^ref, :subscribed, %{channel: &quot;my_channel&quot;}} -&gt; :ok end Redix.command!(client, ~w(PUBLISH my_channel hello) receive do {:redix_pubsub, ^pubsub, ^ref, :message, %{channel: &quot;my_channel&quot;} = properties} -&gt; properties.payload end #=&gt; &quot;hello&quot; Redix.PubSub.unsubscribe(pubsub, &quot;foo&quot;, self()) #=&gt; :ok # We wait for the unsubscription confirmation receive do {:redix_pubsub, ^pubsub, ^ref, :unsubscribed, _} -&gt; :ok end","ref":"Redix.PubSub.html","title":"Redix.PubSub","type":"module"},{"doc":"Subscribes subscriber to the given pattern or list of patterns. Works like subscribe/3 but subscribing subscriber to a pattern (or list of patterns) instead of regular channels. Upon successful subscription to each of the patterns, a message will be sent to subscriber with the following form: {:redix_pubsub, pid, ^subscription_ref, :psubscribed, %{pattern: pattern}} See the documentation for Redix.PubSub for more information about the format of messages. Examples iex&gt; Redix.psubscribe(conn, &quot;ba*&quot;, self()) :ok iex&gt; flush() {:redix_pubsub, ^conn, ^subscription_ref, :psubscribe, %{pattern: &quot;ba*&quot;}} :ok","ref":"Redix.PubSub.html#psubscribe/3","title":"Redix.PubSub.psubscribe/3","type":"function"},{"doc":"Unsubscribes subscriber from the given pattern or list of patterns. This function basically &quot;undoes&quot; what psubscribe/3 does: it unsubscribes subscriber from the given pattern or list of patterns. Upon successful unsubscription from each of the patterns, a message will be sent to subscriber with the following form: {:redix_pubsub, pid, ^subscription_ref, :punsubscribed, %{pattern: pattern}} See the documentation for Redix.PubSub for more information about the format of messages. Examples iex&gt; Redix.punsubscribe(conn, &quot;foo_*&quot;, self()) :ok iex&gt; flush() {:redix_pubsub, ^conn, ^subscription_ref, :punsubscribed, %{pattern: &quot;foo_*&quot;}} :ok","ref":"Redix.PubSub.html#punsubscribe/3","title":"Redix.PubSub.punsubscribe/3","type":"function"},{"doc":"Starts a pub/sub connection to Redis. This function returns {:ok, pid} if the PubSub process is started successfully. The actual TCP/SSL connection to the Redis server may happen either synchronously, before start_link/2 returns, or asynchronously: this behaviour is decided by the :sync_connect option (see below). This function accepts one argument, either a Redis URI as a string or a list of options. Redis URI In case uri_or_opts is a Redis URI, it must be in the form: redis://[:password@]host[:port][/db] Here are some examples of valid URIs: redis://localhost redis://:secret@localhost:6397 redis://example.com:6380/1 Usernames before the password are ignored, so the these two URIs are equivalent: redis://:secret@localhost redis://myuser:secret@localhost The only mandatory thing when using URIs is the host. All other elements (password, port, database) are optional and their default value can be found in the &quot;Options&quot; section below. Options The following options can be used to specify the parameters used to connect to Redis (instead of a URI as described above): :host - (string) the host where the Redis server is running. Defaults to &quot;localhost&quot;. :port - (integer) the port on which the Redis server is running. Defaults to 6379. :password - (string) the password used to connect to Redis. Defaults to nil, meaning no password is used. When this option is provided, all Redix does is issue an AUTH command to Redis in order to authenticate. :database - (integer or string) the database to connect to. Defaults to nil, meaning don&#39;t connect to any database (Redis connects to database 0 by default). When this option is provided, all Redix does is issue a SELECT command to Redis in order to select the given database. :socket_opts - (list of options) this option specifies a list of options that are passed to :gen_tcp.connect/4 when connecting to the Redis server. Some socket options (like :active or :binary) will be overridden by Redix.PubSub so that it functions properly. Defaults to []. :sync_connect - (boolean) decides whether Redix should initiate the TCP connection to the Redis server before or after returning from start_link/2. This option also changes some reconnection semantics; read the &quot;Reconnections&quot; page in the docs for Redix for more information. :backoff_initial - (integer) the initial backoff time (in milliseconds), which is the time that will be waited by the Redix.PubSub process before attempting to reconnect to Redis after a disconnection or failed first connection. See the &quot;Reconnections&quot; page in the docs for Redix for more information. :backoff_max - (integer) the maximum length (in milliseconds) of the time interval used between reconnection attempts. See the &quot;Reconnections&quot; page in the docs for Redix for more information. :exit_on_disconnection - (boolean) if true, the Redix server will exit if it fails to connect or disconnects from Redis. Note that setting this option to true means that the :backoff_initial and :backoff_max options will be ignored. Defaults to false. :log - (keyword list) a keyword list of {action, level} where level is the log level to use to log action. This option is deprecated in favor of Telemetry. See the &quot;Telemetry&quot; section in the module documentation. The possible actions and their default values are: :disconnection (defaults to :error) - logged when the connection to Redis is lost :failed_connection (defaults to :error) - logged when Redix can&#39;t establish a connection to Redis :reconnection (defaults to :info) - logged when Redix manages to reconnect to Redis after the connection was lost :name - Redix is bound to the same registration rules as a GenServer. See the GenServer documentation for more information. :ssl - (boolean) if true, connect through SSL, otherwise through TCP. The :socket_opts option applies to both SSL and TCP, so it can be used for things like certificates. See :ssl.connect/4. Defaults to false. :sentinel - (list of options) exactly the same as the :sentinel options in Redix.start_link/1. :hibernate_after - (integer) if present, the Redix connection process awaits any message for the given number of milliseconds and if no message is received, the process goes into hibernation automatically (by calling :proc_lib.hibernate/3). See t::gen_statem.start_opt/0. Not present by default. :spawn_opt - (options) if present, its value is passed as options to the Redix connection process as in Process.spawn/4. See t::gen_statem.start_opt/0. Not present by default. :debug - (options) if present, the corresponding function in the :sys module is invoked. Not present by default. Examples iex&gt; Redix.PubSub.start_link() {:ok, #PID&lt;...&gt;} iex&gt; Redix.PubSub.start_link(host: &quot;example.com&quot;, port: 9999, password: &quot;secret&quot;) {:ok, #PID&lt;...&gt;} iex&gt; Redix.PubSub.start_link([database: 3], [name: :redix_3]) {:ok, #PID&lt;...&gt;}","ref":"Redix.PubSub.html#start_link/1","title":"Redix.PubSub.start_link/1","type":"function"},{"doc":"Same as start_link/1 but using both a Redis URI and a list of options. In this case, options specified in opts have precedence over values specified by uri. For example, if uri is redix://example1.com but opts is [host: &quot;example2.com&quot;], then example2.com will be used as the host when connecting.","ref":"Redix.PubSub.html#start_link/2","title":"Redix.PubSub.start_link/2","type":"function"},{"doc":"Stops the given pub/sub process. This function is synchronous and blocks until the given pub/sub connection frees all its resources and disconnects from the Redis server. timeout can be passed to limit the amount of time allowed for the connection to exit; if it doesn&#39;t exit in the given interval, this call exits. Examples iex&gt; Redix.PubSub.stop(conn) :ok","ref":"Redix.PubSub.html#stop/2","title":"Redix.PubSub.stop/2","type":"function"},{"doc":"Subscribes subscriber to the given channel or list of channels. Subscribes subscriber (which can be anything that can be passed to send/2) to channels, which can be a single channel or a list of channels. For each of the channels in channels which subscriber successfully subscribes to, a message will be sent to subscriber with this form: {:redix_pubsub, pid, subscription_ref, :subscribed, %{channel: channel}} See the documentation for Redix.PubSub for more information about the format of messages. Examples iex&gt; Redix.PubSub.subscribe(conn, [&quot;foo&quot;, &quot;bar&quot;], self()) {:ok, subscription_ref} iex&gt; flush() {:redix_pubsub, ^conn, ^subscription_ref, :subscribed, %{channel: &quot;foo&quot;}} {:redix_pubsub, ^conn, ^subscription_ref, :subscribed, %{channel: &quot;bar&quot;}} :ok","ref":"Redix.PubSub.html#subscribe/3","title":"Redix.PubSub.subscribe/3","type":"function"},{"doc":"Unsubscribes subscriber from the given channel or list of channels. This function basically &quot;undoes&quot; what subscribe/3 does: it unsubscribes subscriber from the given channel or list of channels. Upon successful unsubscription from each of the channels, a message will be sent to subscriber with the following form: {:redix_pubsub, pid, ^subscription_ref, :unsubscribed, %{channel: channel}} See the documentation for Redix.PubSub for more information about the format of messages. Examples iex&gt; Redix.unsubscribe(conn, [&quot;foo&quot;, &quot;bar&quot;], self()) :ok iex&gt; flush() {:redix_pubsub, ^conn, ^subscription_ref, :unsubscribed, %{channel: &quot;foo&quot;}} {:redix_pubsub, ^conn, ^subscription_ref, :unsubscribed, %{channel: &quot;bar&quot;}} :ok","ref":"Redix.PubSub.html#unsubscribe/3","title":"Redix.PubSub.unsubscribe/3","type":"function"},{"doc":"","ref":"Redix.PubSub.html#t:connection/0","title":"Redix.PubSub.connection/0","type":"type"},{"doc":"","ref":"Redix.PubSub.html#t:subscriber/0","title":"Redix.PubSub.subscriber/0","type":"type"},{"doc":"Telemetry integration for event tracing, metrics, and logging. Redix connections (both Redix and Redix.PubSub) execute the following Telemetry events: [:redix, :disconnection] - executed when the connection is lost with the Redis server. There are no measurements associated with this event. Metadata are: :reason - the disconnection reason as a Redix.ConnectionError struct. :address - the address the connection was connected to. :connection - the PID or registered name of the Redix connection that emitted the event. [:redix, :failed_connection] - executed when Redix can&#39;t connect to the specified Redis server, either when starting up the connection or after a disconnection. There are no measurements associated with this event. Metadata are: :reason - the disconnection reason as a Redix.ConnectionError struct. :address or :sentinel_address - the address the connection was trying to connect to (either a Redis server or a Redis Sentinel instance). :connection - the PID or registered name of the Redix connection that emitted the event. [:redix, :reconnection] - executed when a Redix connection that had disconnected reconnects to a Redis server. There are no measurements associated with this event. Metadata are: :address - the address the connection successfully reconnected to. :connection - the PID or registered name of the Redix connection that emitted the event. Redix connections execute the following Telemetry events when commands or pipelines of any kind are executed. [:redix, :pipeline] - executed when a pipeline (or command, which is a pipeline with just one command) is successfully sent to the server and a reply comes from the server. Measurements are: :elapsed_time (integer) - the elapsed time that it took to send the pipeline to the server and get a reply. The elapsed time is expressed in the :native time unit. See System.convert_time_unit/3. Metadata are: :connection - the connection that emitted the event. If the connection was registered with a name, the name is used here, otherwise the PID. :commands - the commands sent to the server. This is always a list of commands, so even if you do Redix.command(conn, [&quot;PING&quot;]) than the list of commands will be [[&quot;PING&quot;]]. :start_time - the system time when the pipeline was issued. This could be useful for tracing. The time unit is :native, see System.convert_time_unit/3. [:redix, :pipeline, :error] - executed when there&#39;s an error talking to the server. There are no measurements. Metadata are: :connection - the connection that emitted the event. If the connection was registered with a name, the name is used here, otherwise the PID. :commands - the commands sent to the server. This is always a list of commands, so even if you do Redix.command(conn, [&quot;PING&quot;]) than the list of commands will be [[&quot;PING&quot;]]. :start_time - the system time when the pipeline was issued. This could be useful for tracing. The time unit is :native, see System.convert_time_unit/3. :reason - the error reason. More events might be added in the future and that won&#39;t be considered a breaking change, so if you&#39;re writing a handler for Redix events be sure to ignore events that are not known. All future Redix events will start with the :redix atom, like the ones above. A default handler that logs these events appropriately is provided, see attach_default_handler/0. Otherwise, you can write your own handler to instrument or log events, see the Telemetry page in the docs.","ref":"Redix.Telemetry.html","title":"Redix.Telemetry","type":"module"},{"doc":"Attaches the default Redix-provided Telemetry handler. This function attaches a default Redix-provided handler that logs (using Elixir&#39;s Logger) the following events: [:redix, :disconnection] - logged at the :error level [:redix, :failed_connection] - logged at the :error level [:redix, :reconnection] - logged at the :info level See the module documentation for more information. If you want to attach your own handler, look at the Telemetry page in the documentation. Examples :ok = Redix.Telemetry.attach_default_handler()","ref":"Redix.Telemetry.html#attach_default_handler/0","title":"Redix.Telemetry.attach_default_handler/0","type":"function"},{"doc":"Error in the connection to Redis. This exception represents errors in the connection to Redis: for example, request timeouts, disconnections, and similar. Exception fields This exception has the following public fields: :reason - (atom) the error reason. It can be one of the Redix-specific reasons described in the &quot;Error reasons&quot; section below, or any error reason returned by functions in the :gen_tcp module (see the :inet.posix/0 type. Error reasons The :reason field can assume a few Redix-specific values: :closed: when the connection to Redis is closed (and Redix is reconnecting) and the user attempts to talk to Redis :disconnected: when the connection drops while a request to Redis is in flight. :timeout: when Redis doesn&#39;t reply to the request in time.","ref":"Redix.ConnectionError.html","title":"Redix.ConnectionError","type":"exception"},{"doc":"Callback implementation for Exception.message/1.","ref":"Redix.ConnectionError.html#message/1","title":"Redix.ConnectionError.message/1","type":"function"},{"doc":"","ref":"Redix.ConnectionError.html#t:t/0","title":"Redix.ConnectionError.t/0","type":"type"},{"doc":"Error returned by Redis. This exception represents semantic errors returned by Redis: for example, non-existing commands or operations on keys with the wrong type (INCR not_an_integer).","ref":"Redix.Error.html","title":"Redix.Error","type":"exception"},{"doc":"","ref":"Redix.Error.html#t:t/0","title":"Redix.Error.t/0","type":"type"},{"doc":"Error in parsing data according to the RESP protocol.","ref":"Redix.Protocol.ParseError.html","title":"Redix.Protocol.ParseError","type":"exception"},{"doc":"Redix Fast, pipelined, resilient Redis client for Elixir. Redix is a Redis client written in pure Elixir with focus on speed, correctness, and resiliency (that is, being able to automatically reconnect to Redis in case of network errors). This README refers to the master branch of Redix, not the latest released version on Hex. Make sure to check the documentation for the version you&#39;re using.","ref":"readme.html","title":"Redix","type":"extras"},{"doc":"Idiomatic interface for sending commands to Redis Pipelining Resiliency (automatic reconnections) Pub/Sub SSL Redis Sentinel","ref":"readme.html#features","title":"Redix - Features","type":"extras"},{"doc":"Add the :redix dependency to your mix.exs file. If you plan on connecting to a Redis server over SSL you may want to add the optional :castore dependency as well: defp deps() do [ {:redix, &quot;&gt;= 0.0.0&quot;}, {:castore, &quot;&gt;= 0.0.0&quot;} ] end Then, run mix deps.get in your shell to fetch the new dependencies.","ref":"readme.html#installation","title":"Redix - Installation","type":"extras"},{"doc":"Redix is simple: it doesn&#39;t wrap Redis commands with Elixir functions. It only provides functions to send any Redis command to the Redis server. A Redis command is expressed as a list of strings making up the command and its arguments. Connections are started via start_link/0,1,2: {:ok, conn} = Redix.start_link(host: &quot;example.com&quot;, port: 5000) {:ok, conn} = Redix.start_link(&quot;redis://localhost:6379/3&quot;, name: :redix) Commands can be sent using Redix.command/2,3: Redix.command(conn, [&quot;SET&quot;, &quot;mykey&quot;, &quot;foo&quot;]) #=&gt; {:ok, &quot;OK&quot;} Redix.command(conn, [&quot;GET&quot;, &quot;mykey&quot;]) #=&gt; {:ok, &quot;foo&quot;} Pipelines are just lists of commands sent all at once to Redis for which Redis replies with a list of responses. They can be used in Redix via Redix.pipeline/2,3: Redix.pipeline(conn, [[&quot;INCR&quot;, &quot;foo&quot;], [&quot;INCR&quot;, &quot;foo&quot;], [&quot;INCRBY&quot;, &quot;foo&quot;, &quot;2&quot;]]) #=&gt; {:ok, [1, 2, 4]} Redix.command/2,3 and Redix.pipeline/2,3 always return {:ok, result} or {:error, reason}. If you want to access the result directly and raise in case there&#39;s an error, bang! variants are provided: Redix.command!(conn, [&quot;PING&quot;]) #=&gt; &quot;PONG&quot; Redix.pipeline!(conn, [[&quot;SET&quot;, &quot;mykey&quot;, &quot;foo&quot;], [&quot;GET&quot;, &quot;mykey&quot;]]) #=&gt; [&quot;OK&quot;, &quot;foo&quot;] Resiliency Redix is resilient against network errors. For example, if the connection to Redis drops, Redix will automatically try to reconnect periodically at a given &quot;backoff&quot; interval. Look at the documentation for the Redix module and at the &quot;Reconnections&quot; page in the documentation for more information on the available options and on the exact reconnection behaviour. Redis Sentinel Redix supports Redis Sentinel out of the box. You can specify a list of sentinels to connect to when starting a Redix (or Redix.PubSub) connection. Every time that connection will need to connect to a Redis server (the first time or after a disconnection), it will try to connect to one of the sentinels in order to ask that sentinel for the current primary or a replica. sentinels = [&quot;redis://sent1.example.com:26379&quot;, &quot;redis://sent2.example.com:26379&quot;] {:ok, primary} = Redix.start_link(sentinel: [sentinels: sentinels, group: &quot;main&quot;]) Terminology Redix doesn&#39;t support the use of the terms &quot;master&quot; and &quot;slave&quot; that are usually used with Redis Sentinel. I don&#39;t think those are good terms to use, period. Instead, Redix uses the terms &quot;primary&quot; and &quot;replica&quot;. If you&#39;re interested in the discussions around this, this issue in the Redis repository might be interesting to you. Pub/Sub A Redix.PubSub process can be started via Redix.PubSub.start_link/2: {:ok, pubsub} = Redix.PubSub.start_link() Most communication with the Redix.PubSub process happens via Elixir messages (that simulate a Pub/Sub interaction with the pub/sub server). {:ok, pubsub} = Redix.PubSub.start_link() Redix.PubSub.subscribe(pubsub, &quot;my_channel&quot;, self()) #=&gt; {:ok, ref} Confirmation of subscriptions is delivered as an Elixir message: receive do {:redix_pubsub, ^pubsub, ^ref, :subscribed, %{channel: &quot;my_channel&quot;}} -&gt; :ok end If someone publishes a message on a channel we&#39;re subscribed to: receive do {:redix_pubsub, ^pubsub, ^ref, :message, %{channel: &quot;my_channel&quot;, payload: &quot;hello&quot;}} -&gt; IO.puts(&quot;Received a message!&quot;) end","ref":"readme.html#usage","title":"Redix - Usage","type":"extras"},{"doc":"Redix is low-level, but it&#39;s still built to handle most things thrown at it. For many applications, you can avoid pooling with little to no impact on performance. Read the &quot;Real world usage&quot; page in the documentation for more information on this and pooling strategies that work better with Redix.","ref":"readme.html#using-redix-in-the-real-world","title":"Redix - Using Redix in the Real World™","type":"extras"},{"doc":"To run the Redix test suite you will have to have Redis running locally. Redix requires a somewhat complex setup for running tests (because it needs a few instances running, for pub/sub and sentinel). For this reason, in this repository you&#39;ll find a docker-compose.yml file so that you can use Docker and docker-compose to spin up all the necessary Redis instances with just one command. Make sure you have Docker installed and then just run: docker-compose up Now, you&#39;re ready to run tests with the $ mix test command.","ref":"readme.html#contributing","title":"Redix - Contributing","type":"extras"},{"doc":"Redix is released under the MIT license. See the license file.","ref":"readme.html#license","title":"Redix - License","type":"extras"},{"doc":"Reconnections Redix tries to be as resilient as possible. When the connection to Redis drops for some reason, a Redix process will try to reconnect to the Redis server. If there are pending requests to Redix when a disconnection happens, the Redix functions will return {:error, %Redix.ConnectionError{reason: :disconnected}} to the caller. The caller is responsible to retry the request if interested. The first reconnection attempts happens after a backoff interval decided by the :backoff_initial option. If this attempt succeeds, then Redix will start to function normally again. If this attempt fails, then subsequent reconnection attempts are made until one of them succeeds. The backoff interval between these subsequent reconnection attempts is increased exponentially (with a fixed factor of 1.5). This means that the first attempt will be made after n milliseconds, the second one after n * 1.5 milliseconds, the third one after n * 1.5 * 1.5 milliseconds, and so on. Since this growth is exponential, it won&#39;t take many attempts before this backoff interval becomes large: because of this, Redix.start_link/2 also accepts a :backoff_max option. which specifies the maximum backoff interval that should be used. The :backoff_max option can be used to simulate constant backoff after some exponential backoff attempts: for example, by passing backoff_max: 5_000 and backoff_initial: 5_000, attempts will be made regularly every 5 seconds.","ref":"reconnections.html","title":"Reconnections","type":"extras"},{"doc":"The :sync_connect option passed to Redix.start_link/2 decides whether Redix should initiate the TCP connection to the Redis server before or afterRedix.start_link/2 returns. This option also changes the behaviour of Redix when the TCP connection can&#39;t be initiated at all. When :sync_connect is false, then a failed attempt to initially connect to the Redis server is treated exactly as a disconnection: attempts to reconnect are made as described above. This behaviour should be used when Redix is not a vital part of your application: your application should be prepared to handle Redis being down (for example, using the non &quot;bang&quot; variants to issue commands to Redis and handling {:error, _} tuples). When :sync_connect is true, then a failed attempt to initiate the connection to Redis will cause the Redix process to fail and exit. This might be what you want if Redis is vital to your application. If Redis is vital to your application You should use sync_connect: true if Redis is a vital part of your application: for example, if you plan to use a Redix process under your application&#39;s supervision tree, placed before the parts of your application that depend on it in the tree (so that this way, the application won&#39;t be started until a connection to Redis has been established). With sync_connect: true, disconnections after the TCP connection has been established will behave exactly as above (with reconnection attempts at given intervals). However, if your application can&#39;t function properly without Redix, then you want to use exit_on_disconnection: true. With this option, the connection will crash when a disconnection happens. With :sync_connect and :exit_on_disconnection, you can isolate the part of your application that can&#39;t work without Redis under a supervisor and bring that part down when Redix crashes: isolated_children = [ {Redix, sync_connect: true, exit_on_disconnection: true}, MyApp.MyGenServer ] isolated_supervisor = %{ id: MyChildSupervisor, type: :supervisor, start: {Supervisor, :start_link, [isolated_children, [strategy: :rest_for_one]]}, } children = [ MyApp.Child1, isolated_supervisor, MyApp.Child2 ] Supervisor.start_link(children, strategy: :one_for_one)","ref":"reconnections.html#synchronous-or-asynchronous-connection","title":"Reconnections - Synchronous or asynchronous connection","type":"extras"},{"doc":"Real-world usage Redix is a low-level driver, but it&#39;s still built to handle most stuff thrown at it. Redix is built to handle multiple Elixir processes sending commands to Redis through it at the same time. It takes advantage of TCP being a full-duplex protocol (bytes are sent in both directions, often at the same time) so that the TCP stream has bytes flowing in both directions (to and from Redis). For example, if two Elixir processes send a PING command to Redis via Redix.command/2, Redix will send both commands to Redis but will concurrently start listening for the reply to these commands; at a given point, both a PING command as well as the PONG response to a previous PING could be flowing in the TCP stream of the socket that Redix is using. There&#39;s a few different ways to use Redix and to pool connections for better high-load support.","ref":"real-world-usage.html","title":"Real-world usage","type":"extras"},{"doc":"For many applications, a single global Redix instance is enough. This is true especially for applications where requests to Redis are not mapping one-to-one to things like user requests (that is, a request for each user). A common pattern in these cases is to have a named Redix process started under the supervision tree: children = [ {Redix, name: :redix} ] Once Redix is started and registered, you can use it with the given name from anywhere: Redix.command(:redix, [&quot;PING&quot;]) #=&gt; {:ok, &quot;PONG&quot;} Note that this pattern extends to more than one global (named) Redix: for example, you could have a Redix process for handling big and infrequent requests and another one to handle short and frequent requests.","ref":"real-world-usage.html#single-named-redix-instance","title":"Real-world usage - Single named Redix instance","type":"extras"},{"doc":"When you want to have a pool of connections, you can start many connections and register them by name. Say you want to have a pool of five Redis connections. You can start these connections in a supervisor under your supervision tree and then create a wrapper module that calls connections from the pool. The wrapper can use any strategy to choose which connection to use, for example a random strategy. defmodule MyApp.Redix do @pool_size 5 def child_spec(_args) do # Specs for the Redix connections. children = for i &lt;- 0..(@pool_size - 1) do Supervisor.child_spec({Redix, name: :&quot;redix_\#{i}&quot;}, id: {Redix, i}) end # Spec for the supervisor that will supervise the Redix connections. %{ id: RedixSupervisor, type: :supervisor, start: {Supervisor, :start_link, [children, [strategy: :one_for_one]]} } end def command(command) do Redix.command(:&quot;redix_\#{random_index()}&quot;, command) end defp random_index() do rem(System.unique_integer([:positive]), @pool_size) end end You can then start the Redix connections and their supervisor in the application&#39;s supervision tree: def start(_type, _args) do children = [ MyApp.Redix, # ...other children ] Supervisor.start_link(children, strategy: :one_for_one) end And then use the new wrapper in your application: MyApp.Redix.command([&quot;PING&quot;]) #=&gt; {:ok, &quot;PONG&quot;} Caveats of the name-based pool The name-based pool works well enough for many use cases but it has a few caveats. The first one is that the load of requests to Redis is distributed fairly among the connections in the pool, but is not distributed in a &quot;smart&quot; way. For example, you might want to send less requests to connections that are behaving in a worse way, such as slower connections. This avoids bottling up connections that are already slow by sending more requests to them and distributes the load more evenly. The other caveat is that you need to think about possible race conditions when using this kind of pool since every time you issue a command you could be using a different connections. If you issue commands from the same process, things will work since the process will block until it receives a reply so we know that Redis received and processed the command before we can issue a new one. However, if you issue commands from different processes, you can&#39;t be sure of the order that they get processed by Redis. After all, this is often true when doing things from different processes and is not particularly Redix specific.","ref":"real-world-usage.html#name-based-pool","title":"Real-world usage - Name-based pool","type":"extras"},{"doc":"Telemetry Since version v0.10.0, Redix uses Telemetry for instrumentation and for having an extensible way of doing logging. Telemetry is a metrics and instrumentation library for Erlang and Elixir applications that is based on publishing events through a common interface and attaching handlers to handle those events. For more information about the library itself, see its README. Before version v0.10.0, Redix.start_link/1 and Redix.PubSub.start_link/1 supported a :log option to control logging. For example, if you wanted to log disconnections at the :error level and reconnections and the :debug level, you would do: Redix.start_link(log: [disconnection: :error, reconnection: :debug]) The :log option is now deprecated in favour of either using the default Redix event handler or writing your own. For information on the Telemetry events that Redix emits, see Redix.Telemetry.","ref":"telemetry.html","title":"Telemetry","type":"extras"},{"doc":"If you want control on how Redix events are logged or on what level they&#39;re logged at, you can use your own event handler. For example, you can create a module to handle these events: defmodule MyApp.RedixTelemetryHandler do require Logger def handle_event([:redix, event], _measurements, metadata, _config) do case event do :disconnection -&gt; human_reason = Exception.message(metadata.reason) Logger.warn(&quot;Disconnected from \#{metadata.address}: \#{human_reason}&quot;) :failed_connection -&gt; human_reason = Exception.message(metadata.reason) Logger.warn(&quot;Failed to connect to \#{metadata.address}: \#{human_reason}&quot;) :reconnection -&gt; Logger.debug(&quot;Reconnected to \#{metadata.address}&quot;) end end end Once you have a module like this, you can attach it when your application starts: events = [ [:redix, :disconnection], [:redix, :failed_connection], [:redix, :reconnection] ] :telemetry.attach_many( &quot;my-redix-log-handler&quot;, events, &amp;MyApp.RedixTelemetryHandler.handle_event/4, :config_not_needed_here )","ref":"telemetry.html#writing-your-own-handler","title":"Telemetry - Writing your own handler","type":"extras"},{"doc":"Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Fix default option replacement for SSL in OTP 22.2. Allow :gen_statem.start_link/3,4 options in Redix.start_link/2 and Redix.PubSub.start_link/2. Change default SSL depth from 2 to 3 (see this issue).","ref":"changelog.html#v0-10-5","title":"Changelog - v0.10.5","type":"extras"},{"doc":"Bug fixes and improvements Fix the default Telemetry handler for Redis Sentinel events (wasn&#39;t properly fixed in v0.10.3). Fix a compile-time warning about the castore library.","ref":"changelog.html#v0-10-4","title":"Changelog - v0.10.4","type":"extras"},{"doc":"Bug fixes and improvements Use more secure SSL default options and optionally use castore if available as a certificate store. Fix the default Telemetry handler for Redis Sentinel events.","ref":"changelog.html#v0-10-3","title":"Changelog - v0.10.3","type":"extras"},{"doc":"Bug fixes and improvements Allow a discarded username when using Redis URIs. Fix the Redix.command/0 type which was [binary()] but which should have been [String.Chars.t()] since we call to_string/1 on each command.","ref":"changelog.html#v0-10-2","title":"Changelog - v0.10.2","type":"extras"},{"doc":"Bug fixes and improvements Improve password checking in Redis URIs. Fix a bug when naming Redix connections with something other than a local name.","ref":"changelog.html#v0-10-1","title":"Changelog - v0.10.1","type":"extras"},{"doc":"Bug fixes and improvements Add support for Telemetry and publish the following events: [:redix, :pipeline], [:redix, :pipeline, :error], [:redix, :disconnection], [:redix, :reconnection], [:redix, failed_connection]. Deprecate the :log option in Redix.start_link/1 and Redix.PubSub.start_link/1 in favour of Telemetry events and a default log handler that can be activated with Redix.Telemetry.attach_default_handler/0. See the documentation for Redix.Telemetry. This is a hard deprecation that shows a warning. Support for the :log option will be removed in the next version. Fix a few minor bugs in Redix.PubSub.","ref":"changelog.html#v0-10-0","title":"Changelog - v0.10.0","type":"extras"},{"doc":"Bug fixes and improvements Fix a bug related to quickly reconnecting PIDs in Redix.PubSub. Improve error messages here and there.","ref":"changelog.html#v0-9-3","title":"Changelog - v0.9.3","type":"extras"},{"doc":"Bug fixes and improvements Add support for URLs with the rediss scheme. Fix a bug where we used the wrong logging level in some places.","ref":"changelog.html#v0-9-2","title":"Changelog - v0.9.2","type":"extras"},{"doc":"Bug fixes and improvements Fix a bad return type from a gen_statem callback (#120). Improve logging for Redis Sentinel.","ref":"changelog.html#v0-9-1","title":"Changelog - v0.9.1","type":"extras"},{"doc":"Breaking changes Bring Redix.PubSub into Redix. Pub/Sub functionality lived in a separate library, redix_pubsub. Now, that functionality has been moved into Redix. This means that if you use redix_pubsub and upgrade your Redix version to 0.9, you will use the redix_pubsub version of Redix.PubSub. If you also upgrade your redix_pubsub version, redix_pubsub will warn and avoid compiling Redix.PubSub so you can use the latest version in Redix. In general, if you upgrade Redix to 0.9 or later just drop the redix_pubsub dependency and make sure your application works with the latest Redix.PubSub API (the message format changed slightly in recent versions). Add support for Redis Sentinel. Don&#39;t raise Redix.Error errors on non-bang variants of functions. This means that for example Redix.command/3 won&#39;t raise a Redix.Error exception in case of Redis errors (like wront typing) and will return that error instead. In general, if you&#39;re pattern matching on {:error, _} to handle connection errors (for example, to retry after a while), now specifically match on {:error, %Redix.ConnectionError{}}. If you want to handle all possible errors the same way, keep matching on {:error, _}. Bug fixes and improvements Fix a bug that wouldn&#39;t let you use Redis URIs without host or port. Don&#39;t ignore the :timeout option when connecting to Redis.","ref":"changelog.html#v0-9-0","title":"Changelog - v0.9.0","type":"extras"},{"doc":"Bug fixes and improvements Fix an error when setting up SSL buffers (#106).","ref":"changelog.html#v0-8-2","title":"Changelog - v0.8.2","type":"extras"},{"doc":"Bug fixes and improvements Re-introduce start_link/2 with two lists of options, but deprecate it. It will be removed in the next Redix version.","ref":"changelog.html#v0-8-1","title":"Changelog - v0.8.1","type":"extras"},{"doc":"Breaking changes Drop support for Elixir &lt; 1.6. Unify start_link options: there&#39;s no more separation between &quot;Redis options&quot; and &quot;connection options&quot;. Now, all the options are passed in together. You can still pass a Redis URI as the first argument. This is a breaking change because now calling start_link/2 with two kewyord lists breaks. Note that start_link/2 with two keyword lists still works, but emits a warning and is deprecated. Bug fixes and improvements Rewrite the connection using gen_statem in order to drop the dependency to Connection. Add Redix.transaction_pipeline/3 and Redix.transaction_pipeline!/3. Use a timeout when connecting to Redis (which sometimes could get stuck). Add support for SSL 🔐 Add Redix.noreply_command/3 and Redix.noreply_pipeline/3 (plus their bang ! variants).","ref":"changelog.html#v0-8-0","title":"Changelog - v0.8.0","type":"extras"},{"doc":"Add support for Unix domain sockets by passing host: {:local, path}.","ref":"changelog.html#v0-7-1","title":"Changelog - v0.7.1","type":"extras"},{"doc":"Breaking changes Drop support for Elixir &lt; 1.3. Remove Redix.format_error/1. Bug fixes and improvements Add Redix.child_spec/1 for use with the child spec changes in Elixir 1.5.","ref":"changelog.html#v0-7-0","title":"Changelog - v0.7.0","type":"extras"},{"doc":"Fix some deprecation warnings around String.to_char_list/1.","ref":"changelog.html#v0-6-1","title":"Changelog - v0.6.1","type":"extras"},{"doc":"Breaking changes Start using Redix.ConnectionError when returning errors instead of just an atom. This is a breaking change since now Redix.command/2 and the other functions return {:error, %Redix.ConnectionError{reason: reason}} instead of {:error, reason}. If you&#39;re matching on specific error reasons, make sure to update your code; if you&#39;re formatting errors through Redix.format_error/1, you can now use Exception.message/1 on the Redix.ConnectionError structs.","ref":"changelog.html#v0-6-0","title":"Changelog - v0.6.0","type":"extras"},{"doc":"Fix some TCP error handling during the connection setup phase.","ref":"changelog.html#v0-5-2","title":"Changelog - v0.5.2","type":"extras"},{"doc":"Fix Redix.stop/1 to be synchronous and not leave zombie processes.","ref":"changelog.html#v0-5-1","title":"Changelog - v0.5.1","type":"extras"},{"doc":"Drop support for Elixir &lt; 1.2 and OTP 17 or earlier.","ref":"changelog.html#v0-5-0","title":"Changelog - v0.5.0","type":"extras"},{"doc":"Add @lexmag to the maintainers :tada: Handle timeouts nicely by returning {:error, :timeout} instead of exiting (which is the default GenServer behaviour). Remove support for specifying a maximum number of reconnection attempts when connecting to Redis (it was the :max_reconnection_attempts option). Use exponential backoff when reconnecting. Don&#39;t reconnect right away after the connection to Redis is lost, but wait for a cooldown time first. Add support for :backoff_initial and :backoff_max options in Redix.start_link/2. These options are used for controlling the backoff behaviour of a Redix connection. Add support for the :sync_connect option when connecting to Redis. Add support for the :exit_on_disconnection option when connecting to Redis. Add support for the :log option when connecting to Redis. Raise ArgumentError exceptions instead of Redix.ConnectionError exceptions for stuff like empty commands. Raise Redix.Error exceptions from Redix.command/3 instead of returning them wrapped in {:error, _}. Expose Redix.format_error/1. Add a &quot;Reconnections&quot; page in the documentation. Extract the Pub/Sub functionality into a separate project.","ref":"changelog.html#v0-4-0","title":"Changelog - v0.4.0","type":"extras"},{"doc":"Fixed a bug in the integer parsing in Redix.Protocol.","ref":"changelog.html#v0-3-6","title":"Changelog - v0.3.6","type":"extras"},{"doc":"Redix.Protocol now uses continuations under the hood for a faster parsing experience. A bug in Redix.Protocol that caused massive memory leaks was fixed. This bug originated upstream in Elixir itself, and I submitted a fix for it here. Some improvements were made to error reporting in the Redix logging.","ref":"changelog.html#v0-3-5","title":"Changelog - v0.3.5","type":"extras"},{"doc":"Fix a bug in the connection that was replacing the provided Redis password with :redacted upon successful connection, making it impossible to reconnect in case of failure (because of the original password now being unavailable).","ref":"changelog.html#v0-3-4","title":"Changelog - v0.3.4","type":"extras"},{"doc":"Fix basically the same bug that was almost fixed in v0.3.2, but this time for real!","ref":"changelog.html#v0-3-3","title":"Changelog - v0.3.3","type":"extras"},{"doc":"Fix a bug in the protocol that failed to parse integers in some cases.","ref":"changelog.html#v0-3-2","title":"Changelog - v0.3.2","type":"extras"},{"doc":"Restructure the Redix architecture to use two Elixir processes per connection instead of one (a process that packs commands and sends them on the socket and a process that listens from the socket and replies to waiting clients); this should speed up Redix when it comes to multiple clients concurrently issueing requests to Redis.","ref":"changelog.html#v0-3-1","title":"Changelog - v0.3.1","type":"extras"},{"doc":"Breaking changes Change the behaviour for an empty list of command passed to Redix.pipeline/2 (Redix.pipeline(conn, [])), which now raises a Redix.ConnectionError complaining about the empty command. Before this release, the behaviour was just a connection timeout. Change the behaviour of empty commands passed to Redix.command/2 or Redix.pipeline/2 (for example, Redix.command(conn, []) or Redix.pipeline(conn, [[&quot;PING&quot;], []])); empty commands now return {:error, :empty_command}. The previous behaviour was just a connection timeout. Remove Redix.start_link/1 in favour of just Redix.start_link/2: now Redis options are separated from the connection options. Redis options can be passed as a Redis URI as well. Bug fixes and improvements Change the error messages for most of the Redix.ConnectionError exceptions from simple atoms to more meaningful messages.","ref":"changelog.html#v0-3-0","title":"Changelog - v0.3.0","type":"extras"},{"doc":"Fix a bug with single-element lists, that were parsed as single elements (and not lists with a single element in them) by Redix.Protocol.parse_multi/2. See whatyouhide/redix#11.","ref":"changelog.html#v0-2-1","title":"Changelog - v0.2.1","type":"extras"},{"doc":"Rename Redix.NetworkError to Redix.ConnectionError (as it&#39;s more generic and more flexible). Add support for PubSub. The following functions have been added to the Redix module: Redix.subscribe/4 Redix.subscribe!/4 Redix.psubscribe/4 Redix.psubscribe!/4 Redix.unsubscribe/4 Redix.unsubscribe!/4 Redix.punsubscribe/4 Redix.punsubscribe!/4 Redix.pubsub?/2","ref":"changelog.html#v0-2-0","title":"Changelog - v0.2.0","type":"extras"},{"doc":"Initial release.","ref":"changelog.html#v0-1-0","title":"Changelog - v0.1.0","type":"extras"}]